@@ -1,4 +1,4 @@
-// Copyright (c) 2015-2020 Georg Brandl.  Licensed under the Apache License,
+// Copyright (c) 2015-2021 Georg Brandl.  Licensed under the Apache License,
 // Version 2.0 <LICENSE-APACHE or http://www.apache.org/licenses/LICENSE-2.0>
 // or the MIT license <LICENSE-MIT or http://opensource.org/licenses/MIT>, at
 // your option. This file may not be copied, modified, or distributed except
@@ -38,6 +38,7 @@ enum Global {
     Frozenset,   // builtins/__builtin__.frozenset
     Bytearray,   // builtins/__builtin__.bytearray
     List,        // builtins/__builtin__.list
+    Int,         // builtins/__builtin__.int
     Encode,      // _codecs.encode
     Other,       // anything else (may be a classobj that is later discarded)
 }
@@ -69,22 +70,49 @@ enum Value {
     Dict(Vec<(Value, Value)>),
 }
 
+/// Options for deserializing.
+#[derive(Clone, Debug, Default)]
+pub struct DeOptions {
+    decode_strings: bool,
+    replace_unresolved_globals: bool,
+}
+
+impl DeOptions {
+    /// Construct with default options:
+    ///
+    /// - don't decode strings saved as STRING opcodes (only protocols 0-2) as UTF-8
+    /// - don't replace unresolvable globals by `None`
+    pub fn new() -> Self {
+        Default::default()
+    }
+
+    /// Activate decoding strings saved as STRING.
+    pub fn decode_strings(mut self) -> Self {
+        self.decode_strings = true;
+        self
+    }
+
+    /// Activate replacing unresolved globals by `None`.
+    pub fn replace_unresolved_globals(mut self) -> Self {
+        self.replace_unresolved_globals = true;
+        self
+    }
+}
+
 /// Decodes pickle streams into values.
 pub struct Deserializer<R: Read> {
     rdr: BufReader<R>,
+    options: DeOptions,
     pos: usize,
     value: Option<Value>,                  // next value to deserialize
     memo: BTreeMap<MemoId, (Value, i32)>,  // pickle memo (value, number of refs)
     stack: Vec<Value>,                     // topmost items on the stack
     stacks: Vec<Vec<Value>>,               // items further down the stack, between MARKs
-    decode_strings: bool,                  // protocol specific switch
 }
 
 impl<R: Read> Deserializer<R> {
-    /// Construct a new Deserializer.  The second argument decides whether
-    /// strings (STRING opcodes, saved only by protocols 0-2) are decoded as
-    /// UTF-8 strings or left as byte vectors.
-    pub fn new(rdr: R, decode_strings: bool) -> Deserializer<R> {
+    /// Construct a new Deserializer.
+    pub fn new(rdr: R, options: DeOptions) -> Deserializer<R> {
         Deserializer {
             rdr: BufReader::new(rdr),
             pos: 0,
@@ -92,10 +120,63 @@ impl<R: Read> Deserializer<R> {
             memo: BTreeMap::new(),
             stack: Vec::with_capacity(128),
             stacks: Vec::with_capacity(16),
-            decode_strings,
+            options,
         }
     }
 
+    /// Reset internal state, allowing reading multiple pickle dump calls from
+    /// a single stream.
+    ///
+    /// By default `(value_)from_reader` closes the input stream. It is possible
+    /// to deserialize multiple pickle objects from a single stream by
+    /// implementing a custom reader and resetting the internal state before
+    /// reading the next value.
+    ///
+    /// # Example
+    ///
+    /// Using `reset_memo` inside a custom deserializer to deserialize multiple
+    /// objects from a single stream.
+    ///
+    /// ```
+    /// # use std::io::Read;
+    /// # use serde_pickle::{Deserializer, Result, DeOptions};
+    /// # use serde::Deserialize;
+    /// struct PickleReader<R: Read + Sized>
+    /// {
+    ///     de: Deserializer<R>,
+    /// }
+    ///
+    /// impl<R: Read + Sized> PickleReader<R>
+    /// {
+    ///    fn new(reader: R) -> PickleReader<R> {
+    ///        PickleReader {
+    ///            de: Deserializer::new(reader, DeOptions::new()),
+    ///        }
+    ///    }
+    ///
+    ///    pub fn read_object<'de, T: Deserialize<'de>>(&mut self) -> Result<T> {
+    ///        self.de.reset_memo();
+    ///        let value = Deserialize::deserialize(&mut self.de)?;
+    ///        Ok(value)
+    ///    }
+    /// }
+    ///
+    /// let input = vec![0x80, 0x04, 0x95, 0x09, 0x00, 0x00, 0x00, 0x00,
+    ///                  0x00, 0x00, 0x00, 0x8c, 0x05, 0x48, 0x65, 0x6c,
+    ///                  0x6c, 0x6f, 0x94, 0x2e, 0x80, 0x04, 0x95, 0x0a,
+    ///                  0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x8c,
+    ///                  0x06, 0x70, 0x69, 0x63, 0x6b, 0x6c, 0x65, 0x94,
+    ///                  0x2e, 0x00];
+    /// let mut reader = PickleReader::new(std::io::Cursor::new(input.as_slice()));
+    /// let string1: String = reader.read_object().unwrap();
+    /// let string2: String = reader.read_object().unwrap();
+    /// assert_eq!(&string1, "Hello");
+    /// assert_eq!(&string2, "pickle");
+    /// ```
+    pub fn reset_memo(&mut self) {
+        self.memo.clear();
+    }
+
     /// Decode a Value from this pickle.  This is different from going through
     /// the generic serde `deserialize`, since it preserves some types that are
     /// not in the serde data model, such as big integers.
@@ -279,6 +360,10 @@ impl<R: Read> Deserializer<R> {
                     let decoded = self.decode_unicode(string)?;
                     self.stack.push(decoded);
                 }
+                BYTEARRAY8 => {
+                    let string = self.read_u64_prefixed_bytes()?;
+                    self.stack.push(Value::Bytes(string));
+                }
 
                 // Tuples
                 EMPTY_TUPLE => self.stack.push(Value::Tuple(Vec::new())),
@@ -416,7 +501,7 @@ impl<R: Read> Deserializer<R> {
                     self.stack.push(state);
                 }
 
-                // Unsupported (mostly class instance building) opcodes
+                // Unsupported opcodes
                 code => return self.error(ErrorCode::Unsupported(code as char))
             }
         }
@@ -702,7 +787,7 @@ impl<R: Read> Deserializer<R> {
 
     // Decode a string - either as Unicode or as bytes.
     fn decode_string(&self, string: Vec<u8>) -> Result<Value> {
-        if self.decode_strings {
+        if self.options.decode_strings {
             self.decode_unicode(string)
         } else {
             Ok(Value::Bytes(string))
@@ -794,6 +879,8 @@ impl<R: Read> Deserializer<R> {
                 Value::Global(Global::List),
             (b"__builtin__", b"bytearray") | (b"builtins", b"bytearray") =>
                 Value::Global(Global::Bytearray),
+            (b"__builtin__", b"int") | (b"builtins", b"int") =>
+                Value::Global(Global::Int),
             _ => Value::Global(Global::Other),
         };
         Ok(value)
@@ -847,6 +934,15 @@ impl<R: Read> Deserializer<R> {
                     _ => self.error(ErrorCode::InvalidValue("list() arg".into())),
                 }
             }
+            Value::Global(Global::Int) => {
+                match self.resolve(argtuple.pop()) {
+                    Some(Value::Int(integer)) => {
+                        self.stack.push(Value::Int(integer));
+                        Ok(())
+                    }
+                    _ => self.error(ErrorCode::InvalidValue("int() arg".into())),
+                }
+            }
             Value::Global(Global::Encode) => {
                 // Byte object encoded as _codecs.encode(x, 'latin1')
                 match self.resolve(argtuple.pop()) {  // Encoding, always latin1
@@ -934,7 +1030,13 @@ impl<R: Read> Deserializer<R> {
             Value::MemoRef(memo_id) => {
                 self.resolve_recursive(memo_id, (), |slf, (), value| slf.convert_value(value))
             },
-            Value::Global(_) => Err(Error::Syntax(ErrorCode::UnresolvedGlobal)),
+            Value::Global(_) => {
+                if self.options.replace_unresolved_globals {
+                    Ok(value::Value::None)
+                } else {
+                    Err(Error::Syntax(ErrorCode::UnresolvedGlobal))
+                }
+            },
         }
     }
 }
@@ -995,7 +1097,13 @@ impl<'de: 'a, 'a, R: Read> de::Deserializer<'de> for &'a mut Deserializer<R> {
                     slf.deserialize_any(visitor)
                 })
             },
-            Value::Global(_) => Err(Error::Syntax(ErrorCode::UnresolvedGlobal)),
+            Value::Global(_) => {
+                if self.options.replace_unresolved_globals {
+                    visitor.visit_unit()
+                } else {
+                    Err(Error::Syntax(ErrorCode::UnresolvedGlobal))
+                }
+            },
         }
     }
 
@@ -1164,8 +1272,8 @@ impl<'de: 'a, 'a, R: Read> de::MapAccess<'de> for MapAccess<'a, R> {
 
 
 /// Decodes a value from a `std::io::Read`.
-pub fn from_reader<'de, R: io::Read, T: de::Deserialize<'de>>(rdr: R) -> Result<T> {
-    let mut de = Deserializer::new(rdr, false);
+pub fn from_reader<'de, R: io::Read, T: de::Deserialize<'de>>(rdr: R, options: DeOptions) -> Result<T> {
+    let mut de = Deserializer::new(rdr, options);
     let value = de::Deserialize::deserialize(&mut de)?;
     // Make sure the whole stream has been consumed.
     de.end()?;
@@ -1173,29 +1281,33 @@ pub fn from_reader<'de, R: io::Read, T: de::Deserialize<'de>>(rdr: R) -> Result<
 }
 
 /// Decodes a value from a byte slice `&[u8]`.
-pub fn from_slice<'de, T: de::Deserialize<'de>>(v: &[u8]) -> Result<T> {
-    from_reader(io::Cursor::new(v))
+pub fn from_slice<'de, T: de::Deserialize<'de>>(v: &[u8], options: DeOptions) -> Result<T> {
+    from_reader(io::Cursor::new(v), options)
 }
 
 /// Decodes a value from any iterator supported as a reader.
-pub fn from_iter<'de, E: IterReadItem, I: FusedIterator<Item=E>, T: de::Deserialize<'de>>(it: I) -> Result<T> {
-    from_reader(IterRead::new(it))
+pub fn from_iter<'de, E, I, T>(it: I, options: DeOptions) -> Result<T>
+where E: IterReadItem, I: FusedIterator<Item=E>, T: de::Deserialize<'de>
+{
+    from_reader(IterRead::new(it), options)
 }
 
 /// Decodes a value from a `std::io::Read`.
-pub fn value_from_reader<R: io::Read>(rdr: R) -> Result<value::Value> {
-    let mut de = Deserializer::new(rdr, false);
+pub fn value_from_reader<R: io::Read>(rdr: R, options: DeOptions) -> Result<value::Value> {
+    let mut de = Deserializer::new(rdr, options);
     let value = de.deserialize_value()?;
     de.end()?;
     Ok(value)
 }
 
 /// Decodes a value from a byte slice `&[u8]`.
-pub fn value_from_slice(v: &[u8]) -> Result<value::Value> {
-    value_from_reader(io::Cursor::new(v))
+pub fn value_from_slice(v: &[u8], options: DeOptions) -> Result<value::Value> {
+    value_from_reader(io::Cursor::new(v), options)
 }
 
 /// Decodes a value from any iterator supported as a reader.
-pub fn value_from_iter<E: IterReadItem, I: FusedIterator<Item=E>>(it: I) -> Result<value::Value> {
-    value_from_reader(IterRead::new(it))
+pub fn value_from_iter<E, I>(it: I, options: DeOptions) -> Result<value::Value>
+where E: IterReadItem, I: FusedIterator<Item=E>
+{
+    value_from_reader(IterRead::new(it), options)
 }
